using System;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;

namespace myControls
{
    /// <summary>
    /// Describes the properties of the
    /// FeaturedWebPart control
    /// </summary>
    public interface IFeaturedBook
    {
        string BookTitle { get; set;}
        DateTime DatePublished { get; set;}
        decimal Price { get; set;}
    }


    /// <summary>
    /// Custom Editor for FeaturedBookPart
    /// </summary>
    public class FeaturedBookEditorPart : EditorPart
    {
        private TextBox _txtBookTitle;
        private Calendar _calDatePublished;
        private TextBox _txtPrice;
        private CompareValidator _valPrice;

        private string _title = "Featured Book Editor";

        /// <summary>
        /// Create standard title
        /// </summary>
        public override string Title
        {
            get { return _title; }
            set { _title = value; }
        }

        /// <summary>
        /// Utility method which returns the child control
        /// in the case of a GenericWebPart
        /// </summary>
        private Control ControlToEdit
        {
            get
            {
                if (this.WebPartToEdit is GenericWebPart)
                    return ((GenericWebPart)WebPartToEdit).ChildControl;
                else
                    return this.WebPartToEdit;
            }
        }


        /// <summary>
        /// Called when you click OK or Apply to 
        /// apply the Web Part property changes
        /// </summary>
        public override bool ApplyChanges()
        {
            bool success = false;
            EnsureChildControls();
            Page.Validate();
            if (Page.IsValid)
            {
                ((IFeaturedBook)ControlToEdit).BookTitle = _txtBookTitle.Text;
                ((IFeaturedBook)ControlToEdit).DatePublished = _calDatePublished.SelectedDate;
                ((IFeaturedBook)ControlToEdit).Price = decimal.Parse(_txtPrice.Text);
                success = true;
            }
            return success;
        }

        /// <summary>
        /// Callled when the Web Part Framework 
        /// has initialized the Web Part being edited
        /// </summary>
        public override void SyncChanges()
        {
            EnsureChildControls();
            _txtBookTitle.Text = ((IFeaturedBook)ControlToEdit).BookTitle;
            _calDatePublished.SelectedDate = ((IFeaturedBook)ControlToEdit).DatePublished;
            _txtPrice.Text = ((IFeaturedBook)ControlToEdit).Price.ToString();
        }

        /// <summary>
        /// Adds the controls rendered by this Editor Part
        /// to the controls collection.
        /// </summary>
        protected override void CreateChildControls()
        {
            // Add Book Title
            _txtBookTitle = new TextBox();
            _txtBookTitle.ID = "txtBookTitle";
            Controls.Add(_txtBookTitle);

            // Add Date Published
            _calDatePublished = new Calendar();
            _calDatePublished.ID = "calDatePublished";
            Controls.Add(_calDatePublished);

            // Add Price
            _txtPrice = new TextBox();
            _txtPrice.ID = "txtPrice";
            _txtPrice.Columns = 5;
            Controls.Add(_txtPrice);

            // Add Price Validator
            _valPrice = new CompareValidator();
            _valPrice.ID = "valPrice";
            _valPrice.ControlToValidate = _txtPrice.ID;
            _valPrice.Operator = ValidationCompareOperator.DataTypeCheck;
            _valPrice.Type = ValidationDataType.Currency;
            _valPrice.Text = "(Must be currency)";
            Controls.Add(_valPrice);
        }

        /// <summary>
        /// Renders the User Interface for the Editor Part
        /// </summary>
        /// <param name="writer"></param>
        protected override void RenderContents(HtmlTextWriter writer)
        {
            // Render Book Title
            RenderLabel(writer, "Book Title:", _txtBookTitle.ClientID);
            writer.WriteBreak();
            _txtBookTitle.RenderControl(writer);

            writer.WriteBreak();
            writer.WriteBreak();

            // Render Date Published
            RenderLabel(writer, "Date Published:", _calDatePublished.ClientID);
            writer.WriteBreak();
            _calDatePublished.RenderControl(writer);

            writer.WriteBreak();
            writer.WriteBreak();

            // Render Price
            RenderLabel(writer, "Price:", _txtPrice.ClientID);
            _valPrice.RenderControl(writer);
            writer.WriteBreak();
            _txtPrice.RenderControl(writer);
            writer.WriteBreak();
        }

        /// <summary>
        /// Renders an accessible Label for the
        /// form fields
        /// </summary>
        private void RenderLabel(HtmlTextWriter writer, string labelText, string associatedControlId)
        {
            writer.AddAttribute(HtmlTextWriterAttribute.For, associatedControlId);
            writer.RenderBeginTag(HtmlTextWriterTag.Label);
            writer.Write(labelText);
            writer.RenderEndTag();
        }


    }
}