using System;
using System.Collections.Generic;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;

namespace myControls
{
    /// <summary>
    /// WebPartChrome which includes multiple dropdown
    /// menus and menu dividers
    /// </summary>
    public class MenuWebPartChrome : WebPartChrome
    {
        /// <summary>
        /// In the constructor, we grab and save
        /// the zone from the base class
        /// </summary>
        public MenuWebPartChrome(WebPartZone zone, WebPartManager manager)
            : base(zone, manager){}


        /// <summary>
        /// The main method for rendering a Web Part.
        /// Here, we take over responsibility for rendering
        /// the title and menu
        /// </summary>
        public override void RenderWebPart(HtmlTextWriter writer, WebPart webPart)
        {
            // Render an enclosing Div
            writer.AddAttribute(HtmlTextWriterAttribute.Id, this.GetWebPartChromeClientID(webPart));
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_chrome");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);

            // Render the title bar
            RenderTitleBar(writer, webPart);

            // Render the Web Part
            base.RenderPartContents(writer, webPart);

            // Close the enclosing Div
            writer.RenderEndTag(); // Close main DIV
        }

        /// <summary>
        /// Renders the title bar area of the chrome.
        /// This is the part that a user can drag
        /// </summary>
        private void RenderTitleBar(HtmlTextWriter writer, WebPart webPart)
        {
            // Render the menu
            RenderMenu(writer, webPart);

            // Create a break
            writer.AddStyleAttribute("clear", "all");
            writer.RenderBeginTag(HtmlTextWriterTag.Br);

            // Render the title bar
            writer.AddAttribute(HtmlTextWriterAttribute.Id, this.GetWebPartTitleClientID(webPart));
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_chromeTitle");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);
            writer.Write(webPart.DisplayTitle);
            writer.RenderEndTag(); // Close title DIV
        }

        /// <summary>
        /// Renders the menus (possibly nested)
        /// </summary>
        private void RenderMenu(HtmlTextWriter writer, WebPart webPart)
        {
            writer.AddStyleAttribute("display", "inline");
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_menu");
            writer.RenderBeginTag(HtmlTextWriterTag.Ul);

            // Get the top-level menu items which are not hidden
            WebPartVerbCollection topLevelVerbs = GetChildVerbs(webPart.Verbs, String.Empty);
            foreach (MenuWebPartVerb verb in topLevelVerbs)
            {
                writer.AddStyleAttribute("float", "left");
                writer.AddAttribute("onmouseover", "menuWebPartZone.showMenu(this)");
                writer.RenderBeginTag(HtmlTextWriterTag.Li);

                RenderMenuRecurse(writer, verb, webPart);
                writer.RenderEndTag();
            }

            writer.RenderEndTag(); // Close Ul
        }

        /// <summary>
        /// The main method for rendering the subMenus.
        /// This method is called recursively so you
        /// can show infinitely nested menus.
        /// </summary>
        private void RenderMenuRecurse(HtmlTextWriter writer, MenuWebPartVerb verb, WebPart webPart)
        {
            WebPartVerbCollection childVerbs = GetChildVerbs(webPart.Verbs, verb.ID);
            if (childVerbs.Count > 0)
            {
                // Renders a menu item that is not a link
                RenderHeaderMenuItem(writer, verb);

                writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_popupMenu");
                writer.AddStyleAttribute("position", "absolute");
                writer.AddStyleAttribute(HtmlTextWriterStyle.Display, "none");
                writer.RenderBeginTag(HtmlTextWriterTag.Ul);
                foreach (MenuWebPartVerb childVerb in childVerbs)
                {
                    writer.AddAttribute("onmouseover", "menuWebPartZone.showMenu(this)");
                    writer.RenderBeginTag(HtmlTextWriterTag.Li);
                    RenderMenuRecurse(writer, childVerb, webPart);
                    writer.RenderEndTag();
                }
                writer.RenderEndTag(); // Close UL
            }
            else
            {
                // Renders a link menu item
                RenderLinkMenuItem(writer, verb, webPart);
            }
        }

        /// <summary>
        /// Renders a menu item that is not a link.
        /// When a user clicks this menu item, it
        /// expands sub-menu items.
        /// </summary>
        private void RenderHeaderMenuItem(HtmlTextWriter writer, MenuWebPartVerb verb)
        {
            // Render divider
            RenderMenuDivider(writer, verb);

            // Render encloding Div
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_menuItem");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);

            // Render verb icon
            RenderMenuIcon(writer, verb);

            // Render the verb text
            writer.Write(verb.Text);

            writer.RenderEndTag(); // Close Div
        }

        /// <summary>
        /// Renders a menu item which causes a postback.
        /// </summary>
        private void RenderLinkMenuItem(HtmlTextWriter writer, MenuWebPartVerb verb, WebPart webPart)
        {
            // Render divider
            RenderMenuDivider(writer, verb);

            // Render Enclosing Div
            writer.AddAttribute(HtmlTextWriterAttribute.Class, "menuWebPartZone_menuItem");
            writer.RenderBeginTag(HtmlTextWriterTag.Div);

            // Render verb icon
            RenderMenuIcon(writer, verb);

            // Render the verb link
            string eventArg = string.Format("partverb:{0}:{1}", verb.ID, webPart.ID);
            string eventRef = this.Zone.Page.ClientScript.GetPostBackClientHyperlink(this.Zone, eventArg);
            writer.AddAttribute(HtmlTextWriterAttribute.Href, eventRef);
            writer.RenderBeginTag(HtmlTextWriterTag.A);
            writer.Write(verb.Text);
            writer.RenderEndTag();

            writer.RenderEndTag(); // Close Div
        }

        /// <summary>
        /// If a menu item has an icon then show it.
        /// </summary>
        /// <remarks>
        /// Notice that we set empty ALT text for
        /// accessibility reasons.
        /// </remarks>
        private void RenderMenuIcon(HtmlTextWriter writer, WebPartVerb verb)
        {
            if (verb.ImageUrl != String.Empty)
            {
                writer.AddAttribute(HtmlTextWriterAttribute.Src, this.Zone.Page.ResolveUrl(verb.ImageUrl));
                writer.AddAttribute(HtmlTextWriterAttribute.Alt, String.Empty);
                writer.AddAttribute(HtmlTextWriterAttribute.Align, "middle");
                writer.RenderBeginTag(HtmlTextWriterTag.Img);
                writer.RenderEndTag();
            }
        }

        /// <summary>
        /// If a menu should display a divider above it,
        /// then show it with a hr tag.
        /// </summary>
        private void RenderMenuDivider(HtmlTextWriter writer, MenuWebPartVerb verb)
        {
            if (verb.hasDivider)
            {
                writer.RenderBeginTag(HtmlTextWriterTag.Hr);
                writer.RenderEndTag();
            }
        }

        /// <summary>
        /// Returns all the verbs that have a certain 
        /// parent verb
        /// </summary>
        private WebPartVerbCollection GetChildVerbs(WebPartVerbCollection verbs, string parentId)
        {
            List<WebPartVerb> children = new List<WebPartVerb>();
            foreach (MenuWebPartVerb verb in verbs)
                if (verb.parentVerbId == parentId)
                    children.Add(verb);
            return new WebPartVerbCollection(children);
        }

    }
}