Imports System
Imports System.Web
Imports System.Data
Imports System.Data.SqlClient
Imports System.Web.Configuration
Imports System.Collections.Generic
 
Namespace AspNetUnleashed
 
    ''' <summary>
    ''' Represents a customer shopping cart and methods
    ''' for working with shopping carts
    ''' </summary>
    Public Class ShoppingCart

        Private Shared ReadOnly _connectionString As String

        ''' <summary>
        ''' Adds a new item to the shopping cart
        ''' </summary>
        Public Shared Sub Insert(ByVal productId As Integer)
            ' Get user name
            Dim userName As String = GetUserName()

            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("ShoppingCartInsert", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@UserName", userName)
            cmd.Parameters.AddWithValue("@ProductId", productId)

            ' Execute command
            Using con
                con.Open()
                cmd.ExecuteNonQuery()
            End Using

            ' Clear session cart
            Dim context As HttpContext = HttpContext.Current
            context.Session.Remove("ShoppingCart")
        End Sub


        ''' <summary>
        ''' When a user transitions from anonymous to authenticated,
        ''' the user's shopping cart is modified to reflect the 
        ''' authenticated user name
        ''' </summary>
        Public Shared Sub AuthenticateCart(ByVal anonymousId As String, ByVal userName As String)
            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("ShoppingCartAuthenticate", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@AnonymousId", anonymousId)
            cmd.Parameters.AddWithValue("@UserName", userName)

            ' Execute command
            Using con
                con.Open()
                cmd.ExecuteNonQuery()
            End Using

            ' Clear session cart
            Dim context As HttpContext = HttpContext.Current
            context.Session.Remove("ShoppingCart")
        End Sub


        ''' <summary>
        ''' Deletes a shopping cart
        ''' </summary>
        Public Shared Sub Delete(ByVal Id As Integer)
            ' Get user name
            Dim userName As String = GetUserName()

            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("ShoppingCartDelete", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@UserName", userName)
            cmd.Parameters.AddWithValue("@Id", Id)

            ' Execute command
            Using con
                con.Open()
                cmd.ExecuteNonQuery()
            End Using

            ' Clear session cart
            Dim context As HttpContext = HttpContext.Current
            context.Session.Remove("ShoppingCart")
        End Sub

        ''' <summary>
        ''' Retrieves shopping cart for current user
        ''' </summary>
        Public Shared Function [Select]() As List(Of CartItem)
            Dim context As HttpContext = HttpContext.Current

            ' Attempt to get shopping cart from session
            Dim results As List(Of CartItem) = CType(context.Session("ShoppingCart"), List(Of CartItem))
            If IsNothing(results) Then
                results = New List(Of CartItem)()
                results = SelectFromDB()
                context.Session("ShoppingCart") = results
            End If
            Return results
        End Function

        ''' <summary>
        ''' Get shopping cart from database
        ''' </summary>
        Private Shared Function SelectFromDB() As List(Of CartItem)
            ' Trace output
            HttpContext.Current.Trace.Warn("Retrieving shopping cart from database")

            ' Get user name
            Dim userName As String = GetUserName()

            ' Initialize command
            Dim con As New SqlConnection(_connectionString)
            Dim cmd As New SqlCommand("ShoppingCartSelect", con)
            cmd.CommandType = CommandType.StoredProcedure

            ' Initialize parameters
            cmd.Parameters.AddWithValue("@UserName", userName)

            Dim results As New List(Of CartItem)()
            Using con
                con.Open()
                Dim reader As SqlDataReader = cmd.ExecuteReader()
                While reader.Read()
                    results.Add(New CartItem(reader))
                End While
            End Using
            Return results
        End Function

        ''' <summary>
        ''' Gets either an anonymous Id or the
        ''' authenticated user name
        ''' </summary>
        Private Shared Function GetUserName() As String
            Dim context As HttpContext = HttpContext.Current
            Dim userName As String = context.Request.AnonymousID
            If context.Request.IsAuthenticated Then
                userName = context.User.Identity.Name
            End If
            Return userName
        End Function

        ''' <summary>
        ''' Retrieve connection string from Web configuration file
        ''' </summary>
        Shared Sub New()
            _connectionString = WebConfigurationManager.ConnectionStrings("Store").ConnectionString
        End Sub

    End Class
 
    ''' <summary>
    ''' Represents an individual item in a shopping cart
    ''' </summary>
    Public Class CartItem
        Private _id As Integer
        Private _name As String
        Private _price As Decimal
        Private _quantity As Integer
 
        Public Property Id() As Integer
        	Get 
            	 Return _id
        	End Get
        	Set (ByVal Value As Integer) 
            	 _id = value
        	End Set
        End Property
 
        Public Property Name() As String
        	Get 
            	 Return _name
        	End Get
        	Set (ByVal Value As String) 
            	 _name = value
        	End Set
        End Property
 
        Public Property Price() As Decimal
        	Get 
            	 Return _price
        	End Get
        	Set (ByVal Value As Decimal) 
            	 _price = value
        	End Set
        End Property
 
        Public Property Quantity() As Integer
        	Get 
            	 Return _quantity
        	End Get
        	Set (ByVal Value As Integer) 
            	 _quantity = value
        	End Set
        End Property
 
        Public Sub New(ByVal reader As SqlDataReader)
            _id = CType(reader("Id"), Integer)
            _name = CType(reader("Name"), String)
            _price = CType(reader("Price"), Decimal)
            _quantity = CType(reader("Quantity"), Integer)
        End Sub
    End Class
End Namespace

